﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.ComponentModel;
using System.Web.UI.WebControls;

[assembly: WebResource("Telerik.Web.UI.Extenders.GridPopupPositionExtender.js", "text/javascript")]
namespace Telerik.Web.UI.Extenders
{
	/// <summary>
	/// Extends RadGrid's PopUp edit form with various position options
	/// </summary>
	[TargetControlType(typeof(Telerik.Web.UI.RadGrid))]

	public class GridPopupPositionExtender : ExtenderControl
	{
		protected override IEnumerable<ScriptDescriptor> GetScriptDescriptors(Control targetControl)
		{
			ScriptBehaviorDescriptor descriptor = new ScriptBehaviorDescriptor("Telerik.Web.UI.Extender.GridPopupPositionExtender", targetControl.ClientID);
			descriptor.AddComponentProperty("grid", targetControl.ClientID);

			descriptor.AddProperty("_id", this.ClientID);
			if (!this.EnablePopupMove) descriptor.AddProperty("_enablePopupMove", this.EnablePopupMove);
			if (this.RelativeTo != PopupPositionRelativeTo.Grid) descriptor.AddProperty("_relativeTo", this.RelativeTo);
			if (this.HorizontalPosition != PopupHorizontalPosition.Left) descriptor.AddProperty("_horizontalPosition", this.HorizontalPosition);
			if (this.VerticalPosition != PopupVerticalPosition.Top) descriptor.AddProperty("_verticalPosition", this.VerticalPosition);
			if (this.HorizontalOffset != 0) descriptor.AddProperty("_horizontalOffset", this.HorizontalOffset);
			if (this.VerticalOffset != 0) descriptor.AddProperty("_verticalOffset", this.VerticalOffset);

			yield return descriptor;
		}

		protected override IEnumerable<ScriptReference> GetScriptReferences()
		{
			yield return new ScriptReference("Telerik.Web.UI.Extenders.GridPopupPositionExtender.js", "Telerik.Web.UI.Extenders");
		}

		/// <summary>
		/// Gets or sets a value indicating whether the user can move the popup window.
		/// </summary>
		[DefaultValue(true), Bindable(true),
		Description("Gets or sets a value indicating whether the user can move the popup window.")]
		public bool EnablePopupMove
		{
			get
			{
				return (bool)(ViewState["EnablePopupMove"] ?? true);
			}
			set
			{
				ViewState["EnablePopupMove"] = value;
			}
		}

		/// <summary>
		/// Gets or sets the object relative to which the popup is positioned.
		/// </summary>
		[DefaultValue(PopupPositionRelativeTo.Grid), Bindable(true),
		Description("Gets or sets the object relative to which the popup is positioned.")]
		public PopupPositionRelativeTo RelativeTo
		{
			get
			{
				return (PopupPositionRelativeTo)(ViewState["RelativeTo"] ?? PopupPositionRelativeTo.Grid);
			}
			set
			{
				ViewState["RelativeTo"] = value;
			}
		}

		/// <summary>
		/// Gets or sets a value indicating the horizontal popup position
		/// </summary>
		[DefaultValue(PopupHorizontalPosition.Left), Bindable(true),
		Description("Gets or sets a value indicating the horizontal popup position")]
		public PopupHorizontalPosition HorizontalPosition
		{
			get
			{
				return (PopupHorizontalPosition)(ViewState["HorizontalPosition"] ?? PopupHorizontalPosition.Left);
			}
			set
			{
				ViewState["HorizontalPosition"] = value;
			}
		}

		/// <summary>
		/// Gets or sets a value indicating the vertical popup position
		/// </summary>
		[DefaultValue(PopupVerticalPosition.Top), Bindable(true),
		Description("Gets or sets a value indicating the vertical popup position")]
		public PopupVerticalPosition VerticalPosition
		{
			get
			{
				return (PopupVerticalPosition)(ViewState["VerticalPosition"] ?? PopupVerticalPosition.Top);
			}
			set
			{
				ViewState["VerticalPosition"] = value;
			}
		}

		/// <summary>
		/// Gets or sets a value (in pixels) indicating the horizontal offset of the popup
		/// relative to the position of the object specified by the RelativeTo property.
		/// This property is applied when HorizontalPosition is different from PopupHorizontalPosition.Center
		/// and specifies the offset from that position.
		/// </summary>
		[DefaultValue(0), Bindable(true),
		Description("Gets or sets a value (in pixels) indicating the horizontal offset of the popup.")]
		public int HorizontalOffset
		{
			get
			{
				return (int)(ViewState["HorizontalOffset"] ?? 0);
			}
			set
			{
				ViewState["HorizontalOffset"] = value;
			}
		}

		/// <summary>
		/// Gets or sets a value (in pixels) indicating the vertical offset of the popup
		/// relative to the position of the object specified by the RelativeTo property.
		/// This property is applied when VerticalPosition is different from PopupVerticalPosition.Middle
		/// and specifies the offset from that position.
		/// </summary>
		[DefaultValue(0), Bindable(true),
		Description("Gets or sets a value (in pixels) indicating the vertical offset of the popup.")]
		public int VerticalOffset
		{
			get
			{
				return (int)(ViewState["VerticalOffset"] ?? 0);
			}
			set
			{
				ViewState["VerticalOffset"] = value;
			}
		}

		/// <summary>
		/// Enumerates the objects the popup can be positioned relative to.
		/// </summary>
		public enum PopupPositionRelativeTo
		{
			/// <summary>
			/// The popup is positioned relative to the RadGrid instance.
			/// </summary>
			Grid = 1,

			/// <summary>
			/// The popup is positioned relative to the edited item for which it is shown.
			/// </summary>
			EditedItem = 2,

			/// <summary>
			/// The popup is positioned relative to the visible browser viewport
			/// </summary>
			Viewport = 3
		}

		/// <summary>
		/// Enumerates the horizontal popup positions
		/// </summary>
		public enum PopupHorizontalPosition
		{
			Left = 1,
			Center = 2,
			Right = 3
		}

		/// <summary>
		/// Enumerates the vertical popup positions
		/// </summary>
		public enum PopupVerticalPosition
		{
			Top = 1,
			Middle = 2,
			Bottom = 3
		}
	}
}