﻿/// <reference name="MicrosoftAjax.js"/>
/// <reference name="Telerik.Web.UI.Common.Core.js" assembly="Telerik.Web.UI"/>

Type.registerNamespace("Telerik.Web.UI.Extender");

Telerik.Web.UI.Extender.GridPopupPositionExtender = function (element) {
	Telerik.Web.UI.Extender.GridPopupPositionExtender.initializeBase(this, [element]);

	this._id = "";
	this._grid = null;
	this._pageLoadDelegate = null;
	this._enablePopupMove = true;
	this._relativeTo = Telerik.Web.UI.Extender.PopupPositionRelativeTo.Grid;
	this._horizontalPosition = Telerik.Web.UI.Extender.PopupHorizontalPosition.Left;
	this._verticalPosition = Telerik.Web.UI.Extender.PopupVerticalPosition.Top;
	this._horizontalOffset = 0;
	this._verticalOffset = 0;
};

Telerik.Web.UI.Extender.GridPopupPositionExtender.prototype = {
	initialize: function () {
		Telerik.Web.UI.Extender.GridPopupPositionExtender.callBaseMethod(this, "initialize");

		this._pageLoadDelegate = Function.createDelegate(this, this._pageLoad);
		Sys.WebForms.PageRequestManager.getInstance().add_pageLoaded(this._pageLoadDelegate);
	},

	dispose: function () {
		Sys.WebForms.PageRequestManager.getInstance().remove_pageLoaded(this._pageLoadDelegate);
		this._popupShowingDelegate = null;

		Telerik.Web.UI.Extender.GridPopupPositionExtender.callBaseMethod(this, "dispose");
	},

	get_grid: function () {
		return this._grid;
	},

	set_grid: function (value) {
		this._grid = value;
	},

	_pageLoad: function (sender, args) {
		var popup = this._getPopup();
		if (popup) {
			if (!this._enablePopupMove) {
				this._disablePopupDrag(popup);
			}

			var _this = this;
			setTimeout(function () {
				_this.positionPopup(popup);
			}, 100);
		}
	},

	_getPopup: function () {
		if (!this._grid) return null;

		var popupIds = eval(this._grid._popUpIds);
		if (popupIds.length > 0) {
			var popup = $get(popupIds[popupIds.length - 1]);
			if (popup) {
				return popup;
			}
		}

		return null;
	},

	positionPopup: function (popup) {
		var position = { x: 0, y: 0 };
		var relativeBox = this._getRelativeBox(popup);

		switch (this._horizontalPosition) {
			case Telerik.Web.UI.Extender.PopupHorizontalPosition.Left:
				position.x = relativeBox.left + this._horizontalOffset;
				break;

			case Telerik.Web.UI.Extender.PopupHorizontalPosition.Center:
				position.x = relativeBox.left + (relativeBox.width - popup.offsetWidth) / 2;
				break;

			case Telerik.Web.UI.Extender.PopupHorizontalPosition.Right:
				position.x = relativeBox.left + relativeBox.width - popup.offsetWidth - this._horizontalOffset;
				break;
		}

		switch (this._verticalPosition) {
			case Telerik.Web.UI.Extender.PopupVerticalPosition.Top:
				position.y = relativeBox.top + this._verticalOffset;
				break;

			case Telerik.Web.UI.Extender.PopupVerticalPosition.Middle:
				position.y = relativeBox.top + (relativeBox.height - popup.offsetHeight) / 2;
				break;

			case Telerik.Web.UI.Extender.PopupVerticalPosition.Bottom:
				position.y = relativeBox.top + relativeBox.height - popup.offsetHeight - this._verticalOffset;
				break;
		}

		$telerik.setLocation(popup, { x: Math.round(position.x), y: Math.round(position.y) });
	},

	_getRelativeBox: function (popup) {
		var relativeBox = null;
		var isRelativeDataDiv = this._grid && this._grid.GridDataDiv && this._grid.GridDataDiv.style.position === "relative";

		switch (this._relativeTo) {
			case Telerik.Web.UI.Extender.PopupPositionRelativeTo.Grid:
				var relativeLocation = $telerik.getLocation(this.get_element());

				if (isRelativeDataDiv) {
					relativeBox = {
						left: this._grid.GridDataDiv.scrollLeft,
						top: this._grid.GridDataDiv.scrollTop,
						width: this.get_element().offsetWidth,
						height: this.get_element().offsetHeight
					};
				}
				else {
					relativeBox = {
						left: relativeLocation.x,
						top: relativeLocation.y,
						width: this.get_element().offsetWidth,
						height: this.get_element().offsetHeight
					};
				}
				break;

			case Telerik.Web.UI.Extender.PopupPositionRelativeTo.EditedItem:
				if (!popup) return null;

				var relativeLocation = $telerik.getLocation(popup.parentNode.parentNode);

				if (isRelativeDataDiv) {
					var dataDivLocation = $telerik.getLocation(this._grid.GridDataDiv);
					relativeLocation.x += this._grid.GridDataDiv.scrollLeft - dataDivLocation.x;
					relativeLocation.y += this._grid.GridDataDiv.scrollTop - dataDivLocation.y;
				}

				relativeBox = {
					left: relativeLocation.x,
					top: relativeLocation.y,
					width: popup.parentNode.parentNode.offsetWidth,
					height: popup.parentNode.parentNode.offsetHeight
				};
				break;

			case Telerik.Web.UI.Extender.PopupPositionRelativeTo.Viewport:
				var viewPortDimensions = this._getViewPortDimensions();

				if (!isRelativeDataDiv) {
					relativeBox = {
						left: viewPortDimensions.left,
						top: viewPortDimensions.top,
						width: viewPortDimensions.width,
						height: viewPortDimensions.height
					};
				}
				else {
					relativeBox = {
						left: this._grid.GridDataDiv.scrollLeft,
						top: this._grid.GridDataDiv.scrollTop,
						width: this.get_element().offsetWidth,
						height: this.get_element().offsetHeight
					};
				}
				break;
		}

		return relativeBox;
	},

	_getViewPortDimensions: function () {
		var scrollXOffset = window.pageXOffset || document.documentElement.scrollLeft || document.body.scrollLeft;
		var scrollYOffset = window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop;

		var viewPortWidth = document.documentElement.clientWidth;
		var viewPortHeight = document.documentElement.clientHeight;

		return {
			left: scrollXOffset,
			top: scrollYOffset,
			width: viewPortWidth,
			height: viewPortHeight
		}
	},

	_disablePopupDrag: function (popup) {
		var header = popup.getElementsByTagName("div")[0];
		header.style.cursor = "default";
		$clearHandlers(header);
	}
};

Telerik.Web.UI.Extender.GridPopupPositionExtender.registerClass("Telerik.Web.UI.Extender.GridPopupPositionExtender", Sys.UI.Behavior);


Telerik.Web.UI.Extender.PopupPositionRelativeTo = function () { };
Telerik.Web.UI.Extender.PopupPositionRelativeTo.prototype = {
	Grid: 1,
	EditedItem: 2,
	Viewport: 3
};
Telerik.Web.UI.Extender.PopupPositionRelativeTo.registerEnum("Telerik.Web.UI.Extender.PopupPositionRelativeTo");

Telerik.Web.UI.Extender.PopupHorizontalPosition = function () { };
Telerik.Web.UI.Extender.PopupHorizontalPosition.prototype = {
	Left: 1,
	Center: 2,
	Right: 3
};
Telerik.Web.UI.Extender.PopupHorizontalPosition.registerEnum("Telerik.Web.UI.Extender.PopupHorizontalPosition");

Telerik.Web.UI.Extender.PopupVerticalPosition = function () { };
Telerik.Web.UI.Extender.PopupVerticalPosition.prototype = {
	Top: 1,
	Middle: 2,
	Bottom: 3
};
Telerik.Web.UI.Extender.PopupVerticalPosition.registerEnum("Telerik.Web.UI.Extender.PopupVerticalPosition");